/* 
 * Copyright (C) 2006-2007 Zimbra, Inc.
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of version 2 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 */

#ifndef E_ZIMBRA_ITEM_H
#define E_ZIMBRA_ITEM_H

#include <libxml/xmlwriter.h>
#include <libical/icaltime.h>
#include "e-zimbra-folder.h"

/* Standard GObject macros */
#define E_TYPE_ZIMBRA_ITEM \
        (e_zimbra_item_get_type ())
#define E_ZIMBRA_ITEM(obj) \
        (G_TYPE_CHECK_INSTANCE_CAST \
        ((obj), E_TYPE_ZIMBRA_ITEM, EZimbraItem))
#define E_ZIMBRA_ITEM_CLASS(cls) \
        (G_TYPE_CHECK_CLASS_CAST \
        ((cls), E_TYPE_ZIMBRA_ITEM, EZimbraItemClass))
#define E_IS_ZIMBRA_ITEM(obj) \
        (G_TYPE_CHECK_INSTANCE_TYPE \
        ((obj), E_TYPE_ZIMBRA_ITEM))
#define E_IS_ZIMBRA_ITEM_CLASS(cls) \
        (G_TYPE_CHECK_CLASS_TYPE \
        ((cls), E_TYPE_ZIMBRA_ITEM))
#define E_ZIMBRA_ITEM_GET_CLASS(obj) \
        (G_TYPE_INSTANCE_GET_CLASS \
        ((obj), E_TYPE_ZIMBRA_ITEM, EZimbraItemClass))

G_BEGIN_DECLS

typedef struct _EZimbraItemRecurrenceRule EZimbraItemRecurrenceRule;

typedef struct _EZimbraItem EZimbraItem;
typedef struct _EZimbraItemClass EZimbraItemClass;
typedef struct _EZimbraItemPrivate EZimbraItemPrivate;

typedef enum
{
        E_ZIMBRA_ITEM_TYPE_MAIL,
        E_ZIMBRA_ITEM_TYPE_APPOINTMENT,
        E_ZIMBRA_ITEM_TYPE_TASK,
        E_ZIMBRA_ITEM_TYPE_CONTACT,
        E_ZIMBRA_ITEM_TYPE_GROUP,
        E_ZIMBRA_ITEM_TYPE_ORGANISATION,
        E_ZIMBRA_ITEM_TYPE_RESOURCE,
        E_ZIMBRA_ITEM_TYPE_CATEGORY,
        E_ZIMBRA_ITEM_TYPE_NOTIFICATION, 
        E_ZIMBRA_ITEM_TYPE_NOTE,
        E_ZIMBRA_ITEM_TYPE_UNKNOWN
} EZimbraItemType;

typedef enum
{
        E_ZIMBRA_ITEM_CHANGE_TYPE_ADD,
        E_ZIMBRA_ITEM_CHANGE_TYPE_UPDATE,
        E_ZIMBRA_ITEM_CHANGE_TYPE_DELETE,
        E_ZIMBRA_ITEM_CHNAGE_TYPE_UNKNOWN
} EZimbraItemChangeType;

typedef enum
{
        E_ZIMBRA_ITEM_STAT_TENTATIVE    = 1<<0,
        E_ZIMBRA_ITEM_STAT_CONFIRMED    = 1<<1,
        E_ZIMBRA_ITEM_STAT_COMPLETED    = 1<<2,
        E_ZIMBRA_ITEM_STAT_NEEDSACTION  = 1<<3,
        E_ZIMBRA_ITEM_STAT_CANCELLED    = 1<<4,
        E_ZIMBRA_ITEM_STAT_INPROCESS    = 1<<5,
        E_ZIMBRA_ITEM_STAT_DRAFT        = 1<<6,
        E_ZIMBRA_ITEM_STAT_FINAL        = 1<<7,
        E_ZIMBRA_ITEM_STAT_NONE         = 1<<31
} EZimbraItemStatus;

typedef enum
{
        E_ZIMBRA_ITEM_ROLE_CHAIR,
        E_ZIMBRA_ITEM_ROLE_REQUIRED_PARTICIPANT,
        E_ZIMBRA_ITEM_ROLE_OPTIONAL_PARTICIPANT,
        E_ZIMBRA_ITEM_ROLE_NON_PARTICIPANT
} EZimbraItemRole;

typedef enum
{
        E_ZIMBRA_ITEM_PART_STAT_NEEDSACTION     = 1<<0,
        E_ZIMBRA_ITEM_PART_STAT_ACCEPTED        = 1<<1,
        E_ZIMBRA_ITEM_PART_STAT_DECLINED        = 1<<2,
        E_ZIMBRA_ITEM_PART_STAT_TENTATIVE       = 1<<3,
        E_ZIMBRA_ITEM_PART_STAT_DELEGATED       = 1<<4,
        E_ZIMBRA_ITEM_PART_STAT_COMPLETED       = 1<<5,
        E_ZIMBRA_ITEM_PART_STAT_INPROCESS       = 1<<6,
        E_ZIMBRA_ITEM_PART_STAT_NONE            = 1<<31
} EZimbraItemPartStatus;

typedef enum
{
        E_ZIMBRA_ITEM_RECURRENCE_FREQUENCY_NONE         = 0,
        E_ZIMBRA_ITEM_RECURRENCE_FREQUENCY_DAILY        = (1 << 0),
        E_ZIMBRA_ITEM_RECURRENCE_FREQUENCY_WEEKLY       = (1 << 1),
        E_ZIMBRA_ITEM_RECURRENCE_FREQUENCY_MONTHLY      = (1 << 2),
        E_ZIMBRA_ITEM_RECURRENCE_FREQUENCY_YEARLY       = (1 << 3)
} EZimbraItemRecurrenceFrequency;

typedef enum
{
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_NONE = 0,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_SUNDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_MONDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_TUESDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_WEDNESDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_THURSDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_FRIDAY,
        E_ZIMBRA_ITEM_RECURRENCE_WEEKDAY_SATURDAY
} EZimbraItemRecurrenceWeekday;

#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_FIRST  "First" 
#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_SECOND "Second" 
#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_THIRD  "Third" 
#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_FOURTH "Fourth" 
#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_FIFTH  "Fifth"
#define E_ZIMBRA_ITEM_RECUR_OCCURRENCE_TYPE_LAST   "Last"

#define E_ZIMBRA_ITEM_BY_DAY_SIZE       364 /* 7 days * 52 weeks */
#define E_ZIMBRA_ITEM_BY_MONTHDAY_SIZE  32
#define E_ZIMBRA_ITEM_BY_YEARDAY_SIZE   367
#define E_ZIMBRA_ITEM_BY_WEEKNO_SIZE    54
#define E_ZIMBRA_ITEM_BY_MONTH_SIZE     13
#define E_ZIMBRA_ITEM_BY_SET_POS_SIZE   367

struct _EZimbraItemRecurrenceRule
{
        EZimbraItemRecurrenceFrequency frequency;
        gint count;
        icaltimetype until;
        gint interval;
        gshort by_day[E_ZIMBRA_ITEM_BY_DAY_SIZE];
        gshort by_month_day[E_ZIMBRA_ITEM_BY_MONTHDAY_SIZE];
        gshort by_year_day[E_ZIMBRA_ITEM_BY_YEARDAY_SIZE];
        gshort by_week_no[E_ZIMBRA_ITEM_BY_WEEKNO_SIZE];
        gshort by_month[E_ZIMBRA_ITEM_BY_MONTH_SIZE];
        gshort by_set_pos[E_ZIMBRA_ITEM_BY_SET_POS_SIZE];
        EZimbraItemRecurrenceWeekday week_start;
};

#define E_ZIMBRA_ITEM_RECUR_END_MARKER  0x7f7f
#define E_ZIMBRA_ITEM_RECUR_MAX_BYTE    0x7f

struct _EZimbraItem
{
        GObject parent;
        EZimbraItemPrivate *priv;
};

struct _EZimbraItemClass
{
        GObjectClass parent_class;
};

/* structures defined to hold contact item fields */
typedef struct
{
        gchar *name_prefix;
        gchar *first_name;
        gchar *middle_name;
        gchar *last_name;
        gchar *name_suffix;
} FullName;

typedef struct
{
        gchar *street_address;
        gchar *location;
        gchar *city;
        gchar *state;
        gchar *postal_code;
        gchar *country;
} PostalAddress;

typedef struct
{
        gchar *service;
        gchar *address;
} IMAddress;

typedef struct
{
        gchar *id;
        gchar *email;
        gchar *name;
} EGroupMember;

typedef struct
{
        gchar *email;
        gchar *display_name;
} EZimbraItemOrganizer;

typedef enum
{
        E_ZIMBRA_ITEM_RANGE_SINGLE,
        E_ZIMBRA_ITEM_RANGE_THISPRIOR,
        E_ZIMBRA_ITEM_RANGE_THISFUTURE
} EZimbraItemRange;

typedef struct
{
        gchar *id;
        gchar *name;
        gchar *item_reference;
        gchar *contentid;
        gchar *contentType;
        gint size;
        gchar *date;
        gchar *data;
} EZimbraItemAttachment;

typedef struct
{
        gchar *email;
        gchar *display_name;
        gboolean status_enabled;
        gchar *delivered_date;
        gchar *opened_date;
        gchar *accepted_date;
        gchar *deleted_date;
        gchar *declined_date;
        gchar *completed_date;
        gchar *undelivered_date;
        EZimbraItemRole role;
        EZimbraItemPartStatus status;
} EZimbraItemRecipient;

typedef enum
{
        E_ZIMBRA_ITEM_NOTIFY_NONE,
        E_ZIMBRA_ITEM_NOTIFY_MAIL
} EZimbraItemReturnNotify;

typedef enum
{
        E_ZIMBRA_ITEM_NONE,
        E_ZIMBRA_ITEM_DELIVERED,
        E_ZIMBRA_ITEM_DELIVERED_OPENED,
        E_ZIMBRA_ITEM_ALL
} EZimbraItemTrack;

typedef struct
{
        gchar *id;
        gchar *type;
        gchar *thread;
} EZimbraItemLinkInfo;

typedef struct
{
        gchar *item_id;
        gchar *ical_id;
        gchar *recur_key;
} EZimbraItemCalId;

GType           e_zimbra_item_get_type          (void);

EZimbraItem *   e_zimbra_item_new_empty         (void);

EZimbraItem *   e_zimbra_item_new_from_soap_parameter
                                                (gpointer cnc,
                                                 EZimbraItemType type,
                                                 xmlNode *node);

EZimbraItemType e_zimbra_item_get_item_type     (EZimbraItem *item);

void            e_zimbra_item_set_item_type     (EZimbraItem *item,
                                                 EZimbraItemType new_type);

const gchar *   e_zimbra_item_get_folder_id     (EZimbraItem *item);

void            e_zimbra_item_set_folder_id     (EZimbraItem *item,
                                                 const gchar *new_id);

const gchar *   e_zimbra_item_get_icalid        (EZimbraItem *item);

void            e_zimbra_item_set_icalid        (EZimbraItem *item,
                                                 const gchar *new_icalid);

const gchar *   e_zimbra_item_get_id            (EZimbraItem *item);

void            e_zimbra_item_set_id            (EZimbraItem *item,
                                                 const gchar *new_id);

const gchar *   e_zimbra_item_get_rev           (EZimbraItem *item);

void            e_zimbra_item_set_rev           (EZimbraItem *item,
                                                 const gchar *new_rev);

gchar *         e_zimbra_item_get_delivered_date(EZimbraItem *item);

void            e_zimbra_item_set_delivered_date(EZimbraItem *item,
                                                 const gchar *new_date);

const icaltimetype *
                e_zimbra_item_get_start_date    (EZimbraItem* item);

void            e_zimbra_item_set_start_date    (EZimbraItem* item,
                                                 const icaltimetype *start_date);

const icaltimetype *
                e_zimbra_item_get_end_date      (EZimbraItem *item);

void            e_zimbra_item_set_end_date      (EZimbraItem *item,
                                                 const icaltimetype *end_date);

const gchar *   e_zimbra_item_get_subject       (EZimbraItem *item);

void            e_zimbra_item_add_detached_item (EZimbraItem *item,
                                                 EZimbraItem *detached_item);

GList *         e_zimbra_item_peek_detached_items
                                                (EZimbraItem * item);

void            e_zimbra_item_set_subject       (EZimbraItem *item,
                                                 const gchar *new_subject);

const gchar *   e_zimbra_item_get_message       (EZimbraItem *item);

void            e_zimbra_item_set_message       (EZimbraItem *item,
                                                 const gchar *new_message);

const gchar *   e_zimbra_item_get_place         (EZimbraItem *item);

void            e_zimbra_item_set_place         (EZimbraItem *item,
                                                 const gchar *new_place);

gboolean        e_zimbra_item_get_completed     (EZimbraItem *item);

void            e_zimbra_item_set_completed     (EZimbraItem *item,
                                                 gboolean new_completed);

gboolean        e_zimbra_item_get_is_allday_event
                                                (EZimbraItem *item);

void            e_zimbra_item_set_is_allday_event
                                                (EZimbraItem *item,
                                                 gboolean is_allday);        

gchar *         e_zimbra_item_get_field_value   (EZimbraItem *item,
                                                 gchar *field_name);

void            e_zimbra_item_set_field_value   (EZimbraItem *item,
                                                 gchar *field_name,
                                                 gchar* field_value);

GList *         e_zimbra_item_get_email_list    (EZimbraItem *item);

void            e_zimbra_item_set_email_list    (EZimbraItem *item,
                                                 GList *email_list);

FullName *      e_zimbra_item_get_full_name     (EZimbraItem *item);

void            e_zimbra_item_set_full_name     (EZimbraItem *item,
                                                 FullName *full_name);

GList *         e_zimbra_item_get_member_list   (EZimbraItem *item);

void            e_zimbra_item_set_member_list   (EZimbraItem *item,
                                                 GList *list);

PostalAddress * e_zimbra_item_get_address       (EZimbraItem *item,
                                                 gchar *address_type);

void            e_zimbra_item_set_address       (EZimbraItem *item,
                                                 gchar *addres_type,
                                                 PostalAddress *address);

GList *         e_zimbra_item_get_im_list       (EZimbraItem *item);

void            e_zimbra_item_set_im_list       (EZimbraItem *item,
                                                 GList *im_list);

GList *         e_zimbra_item_get_categories    (EZimbraItem *item);

void            e_zimbra_item_set_categories    (EZimbraItem *item,
                                                 GList *category_list);

const gchar *   e_zimbra_item_get_to            (EZimbraItem *item);

void            e_zimbra_item_set_to            (EZimbraItem *item,
                                                 const gchar *to);

const gchar *   e_zimbra_item_get_msg_content_type
                                                (EZimbraItem *item);

EZimbraItemStatus
                e_zimbra_item_get_status        (EZimbraItem *item);

void            e_zimbra_item_set_status        (EZimbraItem *item,
                                                 EZimbraItemStatus status);

gchar *         e_zimbra_item_get_content_type  (EZimbraItem *item);

void            e_zimbra_item_set_content_type  (EZimbraItem *item,
                                                 const gchar *content_type);

EZimbraItemLinkInfo *
                e_zimbra_item_get_link_info     (EZimbraItem *item);

void            e_zimbra_item_set_link_info     (EZimbraItem *item,
                                                 EZimbraItemLinkInfo *info);

const gchar *   e_zimbra_item_get_msg_body_id   (EZimbraItem *item);

gint            e_zimbra_item_get_mail_size     (EZimbraItem *item);

void            e_zimbra_item_set_change        (EZimbraItem *item,
                                                 EZimbraItemChangeType change_type,
                                                 gchar *field_name,
                                                 gpointer field_value);

gchar *         e_zimbra_item_get_category_name (EZimbraItem *item);

void            e_zimbra_item_set_category_name (EZimbraItem *item,
                                                 gchar *cateogry_name);

void            e_zimbra_item_set_sendoptions   (EZimbraItem *item,
                                                 gboolean set);

gboolean        e_zimbra_item_get_reply_request (EZimbraItem *item);

void            e_zimbra_item_set_reply_request (EZimbraItem *item,
                                                 gboolean set);

gchar *         e_zimbra_item_get_reply_within  (EZimbraItem *item);

void            e_zimbra_item_set_reply_within  (EZimbraItem *item,
                                                 gchar *reply_within);

EZimbraItemTrack e_zimbra_item_get_track_info   (EZimbraItem *item);

void            e_zimbra_item_set_track_info    (EZimbraItem *item,
                                                 EZimbraItemTrack track_info);

gboolean        e_zimbra_item_get_autodelete    (EZimbraItem *item);

void            e_zimbra_item_set_autodelete    (EZimbraItem *item,
                                                 gboolean set);

EZimbraItemReturnNotify
                e_zimbra_item_get_notify_completed
                                                (EZimbraItem *item);

void            e_zimbra_item_set_notify_completed
                                                (EZimbraItem *item,
                                                 EZimbraItemReturnNotify notify);

EZimbraItemReturnNotify
                e_zimbra_item_get_notify_accepted
                                                (EZimbraItem *item);

void            e_zimbra_item_set_notify_accepted
                                                (EZimbraItem *item,
                                                 EZimbraItemReturnNotify notify);

EZimbraItemReturnNotify
                e_zimbra_item_get_notify_declined
                                                (EZimbraItem *item);

void            e_zimbra_item_set_notify_declined
                                                (EZimbraItem *item,
                                                 EZimbraItemReturnNotify notify);

EZimbraItemReturnNotify
                e_zimbra_item_get_notify_opened (EZimbraItem *item);

void            e_zimbra_item_set_notify_opened (EZimbraItem *item,
                                                 EZimbraItemReturnNotify notify);

EZimbraItemReturnNotify
                e_zimbra_item_get_notify_deleted(EZimbraItem *item);

void            e_zimbra_item_set_notify_deleted(EZimbraItem *item,
                                                 EZimbraItemReturnNotify notify);

gchar *         e_zimbra_item_get_expires       (EZimbraItem *item);

void            e_zimbra_item_set_expires       (EZimbraItem *item,
                                                 gchar *expires);

gchar *         e_zimbra_item_get_delay_until   (EZimbraItem *item);

void            e_zimbra_item_set_delay_until   (EZimbraItem *item,
                                                 gchar *delay_until);

void            e_zimbra_item_free_cal_id       (EZimbraItemCalId *calid);

#define E_ZIMBRA_ITEM_CLASSIFICATION_PUBLIC       "Public"
#define E_ZIMBRA_ITEM_CLASSIFICATION_PRIVATE      "Private"
#define E_ZIMBRA_ITEM_CLASSIFICATION_CONFIDENTIAL "Confidential"

const gchar *   e_zimbra_item_get_classification(EZimbraItem *item);

void            e_zimbra_item_set_classification(EZimbraItem *item,
                                                 const gchar *new_class);

#define E_ZIMBRA_ITEM_FREE_BUSY_STATUS_FREE             "Free"         
#define E_ZIMBRA_ITEM_FREE_BUSY_STATUS_BUSY             "Busy"
#define E_ZIMBRA_ITEM_FREE_BUSY_STATUS_BUSY_TENTATIVE   "Busy_Tentative"
#define E_ZIMBRA_ITEM_FREE_BUSY_STATUS_BUSY_UNAVAILABLE "Busy_Unavailable"

const gchar *   e_zimbra_item_get_free_busy_status
                                                (EZimbraItem *item);

void            e_zimbra_item_set_free_busy_status
                                                (EZimbraItem *item,
                                                 const gchar *new_status);

typedef enum
{
        E_ZIMBRA_ITEM_TRANSPARENCY_UNKNOWN,
        E_ZIMBRA_ITEM_TRANSPARENCY_NONE,
        E_ZIMBRA_ITEM_TRANSPARENCY_TRANSPARENT,
        E_ZIMBRA_ITEM_TRANSPARENCY_OPAQUE
} EZimbraItemTransparency;

EZimbraItemTransparency
                e_zimbra_item_get_transparency  (EZimbraItem *item);

void            e_zimbra_item_set_transparency  (EZimbraItem *item,
                                                 EZimbraItemTransparency transp);

#define E_ZIMBRA_ITEM_PRIORITY_HIGH     "High"
#define E_ZIMBRA_ITEM_PRIORITY_STANDARD "Standard"
#define E_ZIMBRA_ITEM_PRIORITY_LOW      "Low"

const gchar *   e_zimbra_item_get_priority      (EZimbraItem *item);

void            e_zimbra_item_set_priority      (EZimbraItem *item,
                                                 const gchar *new_priority);

const gchar *   e_zimbra_item_get_task_priority (EZimbraItem *item);

void            e_zimbra_item_set_task_priority (EZimbraItem *item,
                                                 const gchar *new_priority);

GSList *        e_zimbra_item_get_recipient_list(EZimbraItem *item);

void            e_zimbra_item_set_recipient_list(EZimbraItem *item,
                                                 GSList *new_recipient_list);

EZimbraItemOrganizer *
                e_zimbra_item_get_organizer     (EZimbraItem *item);

void            e_zimbra_item_set_organizer     (EZimbraItem *item,
                                                 EZimbraItemOrganizer *organizer);

GSList *        e_zimbra_item_get_attach_id_list(EZimbraItem *item);

void            e_zimbra_item_set_attach_id_list(EZimbraItem *item,
                                                 GSList *attach_list);

GSList *        e_zimbra_item_get_recurrence_dates
                                                (EZimbraItem *item);

void            e_zimbra_item_set_recurrence_dates
                                                (EZimbraItem *item,
                                                 GSList *new_recurrence_dates);

GSList *        e_zimbra_item_get_timezone_list (EZimbraItem *item);

GSList *        e_zimbra_item_get_exdate_list   (EZimbraItem *item);

void            e_zimbra_item_set_exdate_list   (EZimbraItem *item,
                                                 GSList *new_exdate_list);

EZimbraItemRecurrenceRule *
                e_zimbra_item_get_rrule         (EZimbraItem *item);
                
void            e_zimbra_item_set_rrule         (EZimbraItem *item,
                                                 EZimbraItemRecurrenceRule *rrule);

const icaltimetype *
                e_zimbra_item_get_rid           (EZimbraItem *item);

void            e_zimbra_item_set_rid           (EZimbraItem *item,
                                                 const icaltimetype *rid);

GSList *        e_zimbra_item_get_attach_id_list(EZimbraItem *item);

void            e_zimbra_item_set_attach_id_list(EZimbraItem *item,
                                                 GSList *attach_list);

void            e_zimbra_item_set_source        (EZimbraItem *item,
                                                 gchar *source);

gint            e_zimbra_item_get_trigger       (EZimbraItem *item);

void            e_zimbra_item_set_trigger       (EZimbraItem *item,
                                                 gint trigger);

gboolean        e_zimbra_item_append_to_soap_message
                                                (EZimbraItem *item,
                                                 EZimbraItemChangeType type,
                                                 xmlTextWriterPtr msg);

G_END_DECLS

#endif /* E_ZIMBRA_ITEM_H */
